import os
import unittest
import bot.utilss.data as data

class Data_Test(unittest.TestCase):
    """Tests for `data.py`."""

    def test_set_data_file(self):
        """tests if the path is set correctly"""
        path_to_set = "C:\\test\\random\\place\\run.json"
        data.set_data_file(path_to_set)
        self.assertTrue(path_to_set == data.data_file)


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase

from bot.duel_links_runtime import DuelLinkRunTimeOptions
from bot.utils.data import read_json_file, write_data_file


class TestDuelLinkRunTimeOptions(TestCase):
    def setUp(self):
        file = r'D:\Sync\OneDrive\Yu-gi-oh_bot\run_at_test.json'
        self.runtimeoptions = DuelLinkRunTimeOptions(file)


    def test_update(self):
        self.runtimeoptions.update()
        self.runtimeoptions.run_now = True
        self.runtimeoptions.dump()
        tmp_data = read_json_file(self.runtimeoptions._file)
        runtimedict = self.runtimeoptions.dump_options()
        self.assertTrue(tmp_data == runtimedict, "Expecting them to be the same")
        tmp_data['run_now'] = False
        write_data_file(tmp_data, self.runtimeoptions._file)
        self.runtimeoptions.update()
        runtimedict = self.runtimeoptions.dump_options()
        self.assertTrue(tmp_data == runtimedict, "Expecting them to be the same")

    def test_dump(self):
        self.runtimeoptions.dump()
        tmp_data = read_json_file(self.runtimeoptions._file)
        runtimedict = self.runtimeoptions.dump_options()
        self.assertTrue(tmp_data == runtimedict, "Expecting them to be the same")

    def test_runtimeerrors(self):
        self.runtimeoptions.update()
        self.runtimeoptions.stop = 'Yes'
        self.assertTrue(self.runtimeoptions.stop != 'Yes', 'Cannot change to type string')
        self.runtimeoptions.next_run_at = 'Try'
        self.assertTrue(self.runtimeoptions.next_run_at != 'Try', 'Cannot change from datetime to string')



import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(formatter)
try:
    from bot.debug_helpers.logger import LastRecordHandler

    last_record = LastRecordHandler()
    logger.addHandler(last_record)
except ImportError:
    pass

logger.addHandler(ch)
logger.debug("Setup Complete")
from ._version import get_versions

__version__ = get_versions()['version']
del get_versions


def fake_decorator(arg1=0, arg2=0, arg3=0):
    def calling_function(__function):
        """
        Fake Decorator
        """
        def wrapper(*args, **kwargs):
            return __function(*args, **kwargs)
        return wrapper
    return calling_function

default_timestamp = 86400


import numpy as np
import cv2
from matplotlib import pyplot as plt
import PIL as Pillow
import os
os.environ['FOR_DISABLE_CONSOLE_CTRL_HANDLER'] = 'T'
from sklearn.cluster import KMeans


class Trainer(object):
    _debug = False
    def __init__(self, query, x=0, y=0):
        self.query = query
        self.xThreshold = x
        self.yThreshold = y
        if type(query) is Pillow.Image.Image:
            self.query = cv2.cvtColor(np.array(self.query), cv2.COLOR_RGB2BGR)
        elif type(query) is np.ndarray:
            self.query = query
        else:
            self.query = cv2.imread(query, 0)
        self.goodMatches = []
        self.images = []
        self.circlePoints = []
        self.kmeans = None
        self.whitequery = None

    def getMatches(self, train, corr):
        img1 = cv2.imread(train, 0)  # trainImage
        img2 = self.query  # queryImage
        # Initiate SIFT detector
        sift = cv2.xfeatures2d.SIFT_create()

        # find the keypoints and descriptors with SIFT
        kp1, des1 = sift.detectAndCompute(img1, None)
        kp2, des2 = sift.detectAndCompute(img2, None)

        # create BFMatcher object
        bf = cv2.BFMatcher()
        try:
            matches = bf.knnMatch(des1, des2, k=2)
        except cv2.error:
            return False
        goodMatches = []
        cluster = []
        for m, n in matches:
            img2_idx = m.trainIdx
            img1_idx = m.queryIdx
            (x1, y1) = kp1[img1_idx].pt
            (x2, y2) = kp2[img2_idx].pt
            #print("Comare %d to %d and %d to %d" % (x1,x2,y1,y2))
            if m.distance < 0.8 * n.distance and y2 > self.yThreshold and x2 < self.xThreshold:
                goodMatches.append([m])
                cluster.append([int(x2), int(y2)])
        if len(cluster) <= corr:
            return False
        self.kmeans = KMeans(n_clusters=1, random_state=0).fit(cluster)
        # print(self.kmeans.cluster_centers_)
        #plt.scatter(*zip(*cluster)),plt.axis([0,480,0,800]),plt.gca().invert_yaxis(),plt.show()
        img3 = cv2.drawMatchesKnn(
            img1, kp1, img2, kp2, goodMatches, None, flags=2)
        self.images.append(img3)
        # if train != 'assets/back__.png':
        self.debug_matcher(img3)
        return True
        # return goodMatches
        # cv2.drawMatchesKnn expects list of lists as matches.

    def debug_matcher(self,img):
        if self._debug:
            plt.imshow(img);plt.show()
    
    def readCircles(self):
        img = cv2.cvtColor(self.query, cv2.COLOR_BGR2GRAY)
        img = cv2.medianBlur(img, 7)
        cimg = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)

        circles = cv2.HoughCircles(img, cv2.HOUGH_GRADIENT, 1, 30,
                                   param1=50, param2=30, minRadius=20, maxRadius=50)
        if circles is None:
            return
        circles = np.uint16(np.around(circles))
        for i in circles[0, :]:
            # draw the outer circle
            if (i[1] < 400):
                continue
            self.circlePoints.append((i[0], i[1]))
            cv2.circle(cimg, (i[0], i[1]), i[2], (0, 255, 0), 2)
            # draw the center of the circle
            cv2.circle(cimg, (i[0], i[1]), 2, (0, 0, 255), 3)
        # plt.imshow(cimg),plt.show()
        self.images.append(cimg)

    def whiteCircles(self):
        self.prepForWhiteCircles()
        img = cv2.cvtColor(self.whitequery, cv2.COLOR_BGR2GRAY)
        cimg = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
        circles = cv2.HoughCircles(img, cv2.HOUGH_GRADIENT, 1, 40,
                                   param1=50, param2=30, minRadius=5, maxRadius=60)
        if circles is None:
            return
        circles = np.uint16(np.around(circles))
        for i in circles[0, :]:
            self.circlePoints.append((i[0], i[1]))
            # draw the outer circle
            cv2.circle(cimg, (i[0], i[1]), i[2], (0, 255, 0), 2)
            # draw the center of the circle
            cv2.circle(cimg, (i[0], i[1]), 2, (0, 0, 255), 3)
        #cv2.imshow("images", np.hstack([cimg]))
        # cv2.waitKey(0)

    def prepForWhiteCircles(self):
        lower, upper = ([215, 215, 215], [255, 255, 255])
        lower = np.array(lower, dtype="uint8")
        upper = np.array(upper, dtype="uint8")

        # find the colors within the specified boundaries and apply
        # the mask
        mask = cv2.inRange(self.query, lower, upper)
        output = cv2.bitwise_and(
            self.query, self.query, mask=mask)  # coding=utf-8
        self.whitequery = output

    def compare(self):
        plot_image = self.images[0]
        for x in range(1, len(self.images)):
            plot_image = np.concatenate((plot_image, self.images[x]), axis=1)
        plt.imshow(plot_image), plt.show()



import time

import cv2
import numpy as np


def loop_scan(fn, **kwargs):
    limit = 3
    doit = True
    l_times = 0
    while doit and l_times <= limit:
        l_times += 1
        doit = fn(**kwargs)
        time.sleep(1)


def mask_image(lower_mask, upper_mask, img):
    """" Masks an image"""
    assert (len(lower_mask) == 3)
    assert (len(upper_mask) == 3)
    color_min = np.array(lower_mask, np.uint8)
    color_max = np.array(upper_mask, np.uint8)
    new_img = cv2.inRange(img, color_min, color_max)
    return new_img


def crop_image(img, left=0, top=0, width=0, height=0):
    left, top, width, height = tuple(np.asanyarray([left, top, width, height], np.uint64).tolist())
    crop_img = img[top:(top + height), left:(left + width)].copy()
    return crop_img



from .provider import Provider
from bot.providers.nox import Nox
from .steam import Steam
import logging

available_providers = [Nox, Steam]


def get_provider(name):
    if name.lower() == 'nox':
        return Nox
    elif name.lower == 'steam':
        return Steam
    else:
        raise Exception("Provider {} does not exist, choose from [{}]".format(name, ','.join(
            [x.__str__() for x in available_providers])))


root = logging.getLogger("bot.providers")
root.setLevel(logging.DEBUG)
formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(formatter)
root.addHandler(ch)



import asyncio
import datetime
import inspect
import json
import os
import pathlib
import sys
import threading
import traceback
from abc import abstractmethod

from apscheduler.jobstores.base import JobLookupError

from bot import logger, default_timestamp
from bot.utils.data import read_json_file, write_data_file
from bot.utils.watcher import SyncWithFile
import time

try:
    from bot.debug_helpers.helpers_decorators import async_calling_function
except ImportError:
    from bot import fake_decorator as async_calling_function


class DuelLinkRunTimeOptions(object):
    _last_run_at = datetime.datetime.fromtimestamp(default_timestamp)

    @property
    def last_run_at(self):
        return self._last_run_at

    @last_run_at.setter
    def last_run_at(self, value):
        if not isinstance(value, datetime.datetime):
            self.runtime_error_options("last_run_at", datetime.datetime, type(value))
            return
        if self._last_run_at == value:
            return
        self._last_run_at = value
        frame = inspect.currentframe()
        logger.info("Value {} modified to {}".format(inspect.getframeinfo(frame).function, value))
        self.timeout_dump()

    _next_run_at = datetime.datetime.fromtimestamp(default_timestamp)

    @property
    def next_run_at(self):
        return self._next_run_at

    @next_run_at.setter
    def next_run_at(self, value):
        if not isinstance(value, datetime.datetime):
            self.runtime_error_options("next_run_at", datetime.datetime, type(value))
            return
        if self._next_run_at == value:
            return
        self._next_run_at = value
        # IMPLEMENTATION: Notify Scheduler of next run
        frame = inspect.currentframe()
        logger.info("Value {} modified to {}".format(inspect.getframeinfo(frame).function, value))
        self.timeout_dump()
        self.handle_option_change('next_run_at')

    _run_now = False

    @property
    def run_now(self):
        return self._run_now

    @run_now.setter
    def run_now(self, value):
        if not isinstance(value, bool):
            self.runtime_error_options("run_now", bool, type(value))
            return
        if self._run_now == value:
            return
        self._run_now = value
        # IMPLEMENTATION: Notify Scheduler to Run Now
        frame = inspect.currentframe()
        logger.info("Value {} modified".format(inspect.getframeinfo(frame).function))
        self.timeout_dump()
        self.handle_option_change('run_now')

    _stop = False

    @property
    def stop(self):
        return self._stop

    @stop.setter
    def stop(self, stop):
        if not isinstance(stop, bool):
            self.runtime_error_options("stop", bool, type(stop))
            return
        if self._stop == stop:
            return
        self._stop = stop
        frame = inspect.currentframe()
        logger.info("Value {} modified".format(inspect.getframeinfo(frame).function))
        self.timeout_dump()
        self.handle_option_change('stop')
        # IMPLEMENTATION: Notify Scheduler to Stop Now

    _battle_calls = {
        "beforeStart": [],
        "afterStart": [],
        "beforeEnd": [],
        "afterEnd": []
    }

    @property
    def battle_calls(self):
        return self._battle_calls

    @battle_calls.setter
    def battle_calls(self, value):
        if not isinstance(value, dict):
            self.runtime_error_options("battle_calls", dict, type(value))
            return
        if self._battle_calls == value:
            return
        self._battle_calls = value
        frame = inspect.currentframe()
        logger.info("Value {} modified".format(inspect.getframeinfo(frame).function))
        self.timeout_dump()
        self.handle_option_change('battle_calls')

    @abstractmethod
    def runtime_error_options(self, option, expecting_type, got_type):
        raise NotImplementedError("runtime_error_options not implemented")

    @abstractmethod
    def timeout_dump(self):
        raise NotImplementedError("timeout_dump not implemented")

    @abstractmethod
    def handle_option_change(self, value):
        raise NotImplementedError("handle_option_change not implemented")


class DuelLinkRunTime(DuelLinkRunTimeOptions):
    _file = None
    _unknown_options = []
    _scheduler = None
    _config = None
    _watcher = None
    _timeout_dump = None
    _executor = None
    _task = None
    _provider = None
    _loop = None
    _run_main = None
    _job = None
    _allow_event_change = True
    _disable_dump = False

    def __init__(self, config, scheduler):
        self._config = config
        self._file = config.get('bot', 'runTimePersistence')
        self._scheduler = scheduler

        self.setUp()
        logger.debug("Watching {} for runTime Options".format(self._file))
        self._watcher = SyncWithFile(self._file)
        self._watcher.settings_modified = self.settings_modified

    def setUp(self):
        self._loop = asyncio.get_event_loop()
        self._task = asyncio.Task(self.periodic())
        if os.path.dirname(self._file) == "":
            self._file = os.path.join(os.getcwd(), self._file)
        pathlib.Path(os.path.dirname(self._file)).mkdir(parents=True, exist_ok=True)
        if os.path.exists(self._file):
            self.update()

    def handle_option_change(self, value):
        if self._provider is None:
            return
        if value == 'stop':
            if self.stop and self._provider.current_thread is not None:
                for x in threading.enumerate():
                    if x == self._provider.current_thread:
                        self._provider.current_thread.do_run = False
                        logger.info("Stopping Bot Execution")
            elif self._provider.current_thread is not None:
                logger.info("Resuming Bot Execution")
            elif self.stop:
                self.stop = False
        if value == 'run_now' and self.run_now:
            logger.info("Forcing run now")
            if self._provider.current_thread is None:
                try:
                    self._scheduler.remove_job(self._job)
                except JobLookupError:
                    pass
                self._scheduler.add_job(self._run_main, id='cron_main_force')
            else:
                logger.debug("Thread is currently running")
            self.run_now = False
        if value == 'next_run_at' and self._allow_event_change:
            try:
                self._scheduler.remove_job(self._job)
            except JobLookupError:
                pass
            self.schedule_next_run()
            next_run_at = self.next_run_at
            self._job = 'cron_main_at_{}'.format(next_run_at.isoformat())
            self._scheduler.add_job(self._run_main, trigger='date', id=self._job,
                                    run_date=next_run_at)

    def get_provider(self):
        return self._provider

    def set_provider(self, provider):
        self._provider = provider

    def looper(self):
        try:
            self._loop.run_until_complete(self._task)
        except asyncio.CancelledError:
            pass

    def settings_modified(self, events):
        self.update()

    def update(self):
        self._unknown_options = []
        try:
            tmp_data = read_json_file(self._file)
        except json.decoder.JSONDecodeError:
            logger.error("runtime file error reading")
            return
        if tmp_data is None:
            self.dump()
            return
        for key, value, in tmp_data.items():
            if key.startswith('_'):
                continue
            if hasattr(self, key):
                setattr(self, key, value)
            else:
                self._unknown_options.append(key)
        if len(self._unknown_options) > 0:
            logger.debug("Unknown options were passed in [{}]".format(','.join(self._unknown_options)))

    def dump_options(self):
        tmpdict = {}
        for attribute in [a for a in dir(self) if not a.startswith('__') \
                and not a.startswith('_') \
                and not inspect.ismethod(getattr(self, a))
        and not inspect.isfunction(getattr(self, a))]:
            # print(attribute, type(getattr(self,attribute)))
            tmpdict[attribute] = getattr(self, attribute)
        return tmpdict

    @async_calling_function(2)
    def dump(self):
        if not self._disable_dump:
            self._watcher.stop_observer()
            tmpdict = self.dump_options()
            logger.debug("Dump Getting Called {}".format(tmpdict))
            write_data_file(tmpdict, self._file)
            self._watcher.start_observer()

    def timeout_dump(self):
        if isinstance(self._timeout_dump, asyncio.TimerHandle):
            self._timeout_dump.cancel()
        self._timeout_dump = self._loop.call_later(5, self.dump)
        logger.debug("Timeout dump Scheduled")

    @staticmethod
    def runtime_error(message):
        logger.error(
            "Runtime error: {}".format(message)
        )

    def runtime_error_options(self, option, expecting_type, got_type):
        mess = "option {} has wrong type associated with it. Fix it, no events will be notified.".format(option)
        self.runtime_error(mess)
        mess = "option {} expecting {} but got {}".format(option, expecting_type, got_type)
        self.runtime_error(mess)

    def schedule_next_run(self):
        if self._watcher.observer:
            self._watcher.stop_observer()
        if self.next_run_at == datetime.datetime.fromtimestamp(default_timestamp):
            self.next_run_at = datetime.datetime.now() + datetime.timedelta(seconds=5)
        elif datetime.datetime.now() > self.next_run_at:
            self.next_run_at = datetime.datetime.now() + datetime.timedelta(seconds=5)
        else:
            next_at = self.next_run_at - datetime.datetime.now()
            self.next_run_at = datetime.datetime.now(
            ) + datetime.timedelta(seconds=next_at.total_seconds())
        self._watcher.start_observer()

    def main(self):
        def schedule_shutdown():
            self._scheduler.shutdown()

        def thread_shutdown():
            self.shutdown()
            schedule_shutdown()

        def handle_exception(e):
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            logger.error(e)
            logger.debug("{} {} {}".format(exc_type, fname, exc_tb.tb_lineno))
            logger.debug(traceback.format_exc())
            logger.fatal("Provider does not have method correctly implemented cannot continue")
            tt = threading.Thread(target=thread_shutdown, args=())
            tt.start()  # (schedule_shutdown, args=(), id='shutdown')

        def in_main():
            self.last_run_at = datetime.datetime.now()
            provider = self.get_provider()
            try:
                if not provider.is_process_running():
                    provider.start_process()
                    provider.wait_for_ui(30)
                    provider.pass_through_initial_screen()
                provider.compare_with_back_button()
                logger.info("main event")
                provider.auto()
            except NotImplementedError as ee:
                handle_exception(ee)
                return
            except AttributeError as ee:
                handle_exception(ee)
                return
            except TypeError as ee:
                handle_exception(ee)
                return
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                logger.debug("{} {} {}".format(exc_type, fname, exc_tb.tb_lineno))
                logger.debug(traceback.format_exc())
            self._watcher.stop_observer()
            self._allow_event_change = False
            self.next_run_at = datetime.datetime.now() + datetime.timedelta(hours=4)
            next_run_at = self.next_run_at
            self._allow_event_change = True
            self._job = 'cron_main_at_{}'.format(next_run_at.isoformat())
            self._scheduler.add_job(in_main, trigger='date', id=self._job,
                                    run_date=next_run_at)
            self._watcher.start_observer()

        self._allow_event_change = False
        self._run_main = in_main
        self._scheduler.add_job(self.looper, args=(), id="looper")
        if self._config.getboolean("bot", "startBotOnStartUp"):
            self.next_run_at = datetime.datetime.now() + datetime.timedelta(seconds=1)
        else:
            self.schedule_next_run()
        next_run_at = self.next_run_at
        self._job = 'cron_main_at_{}'.format(next_run_at.isoformat())
        self._scheduler.add_job(in_main, trigger='date', id=self._job,
                                run_date=next_run_at)
        self._watcher.start_observer()
        self._allow_event_change = True
        logger.info("Tracking %s" % (self._file))
        logger.info('Next run at %s' % (self.next_run_at.isoformat()))

    @asyncio.coroutine
    def periodic(self):
        while True:
            yield from asyncio.sleep(1)

    _shutdown = False

    def shutdown(self):
        """ Waits for the current thread execution to become None or else will not shutdown properly"""
        self._disable_dump = True  # will not write to run time options
        self.stop = True  # signals all long_running operations to not execute, os calls will not occur either
        while self._provider.current_thread is not None:
            time.sleep(5)
        self._task.cancel()
        self._scheduler.shutdown()
        self._shutdown = True

    def __exit__(self):
        self.dump()



import datetime
import logging
import threading
import time
import subprocess

import apscheduler
import os

import cv2
from bot.duel_links_runtime import DuelLinkRunTime
from bot.providers import trainer_matches as tm
from bot.providers.duellinks import DuelLinks, LOW_CORR
from bot.providers.misc import Misc
from bot.providers.actions import Actions


class Provider(DuelLinks, Misc, Actions):
    scheduler = None
    current_job = None  # indicates the current job running
    lock = None
    # indicates that this provider is currently battling
    current_battle = None
    # logger
    root = logging.getLogger("bot.provider")
    assets = None
    predefined = None

    def setUp(self):
        pass

    def __init__(self, scheduler, config, run_time):
        self.scheduler = scheduler
        self._config = config
        self.assets = config.get('locations', 'assets')
        self.lock = threading.Lock()
        self.run_time = run_time  # type: DuelLinkRunTime
        self.setUp()

    def auto(self):
        t = threading.currentThread()
        self.register_thread(t)
        self.root.info("starting auto run through")
        for x in range(0, 8):
            if self.run_time.stop:
                # Leaves a checkpoint when stopped
                self.current_run = x
                break
            self.root.debug("Run through {}".format(x))
            self.compare_with_back_button()
            self.wait_for_ui(1)
            self.swipe_right()
            try:
                self.scan()
            except Exception:
                break
        self.register_thread(None)

    def debug_battle(self):
        self.battle(CheckBattle=self.check_battle)
        # self.CheckBattle()

    def __check_battle_is_running__(self):
        self.root.debug("CHECKING AUTO DUEL STATUS")
        img = self.get_img_from_screen_shot()
        status = self.determine_autoduel_status(img)
        self.root.debug("AUTO_DUEL STATUS: {}".format(status))
        if not status and self.current_battle:
            self.click_auto_duel()
            self.check_battle()

    def check_battle(self, signal_done=False, delay=5):
        self.lock.acquire()
        try:
            self.scheduler.remove_job(self.current_job)
        except apscheduler.jobstores.base.JobLookupError:
            if signal_done:
                self.current_battle = False
                self.lock.release()
                return
        when = datetime.datetime.now() + datetime.timedelta(seconds=delay)
        job_id = 'cron_check_battle_at_%s' % (when.isoformat())
        self.current_job = job_id
        self.scheduler.add_job(self.__check_battle_is_running__, trigger='date',
                               id=job_id, run_date=when)
        self.lock.release()
        self.root.debug(job_id)

    def scan(self):
        raise NotImplementedError("scan not implemented")

    def possible_battle_points(self):
        if self.run_time.stop:
            self.root.info("Received Stopping signal")
            return
        img = self.get_img_from_screen_shot()
        t = tm.Trainer(img)
        t.whiteCircles()
        current_page = self.get_current_page(img)
        logging.debug("Current-Page {}".format(current_page))
        for x, y in t.circlePoints:
            if self.run_time.stop:
                self.root.info("Received Stopping signal")
                break
            yield x, y, current_page

    def wait_for_auto_duel(self):
        self.root.debug("WAITING FOR AUTO-DUEL TO APPEAR")
        word = ''
        while 'Auto-Duel' not in word and 'AutoDuel' not in word and not self.run_time.stop:
            img = self.get_img_from_screen_shot()
            area = img.crop(self.auto_duel_box)
            try:
                word = Provider.img_to_string(area, "Auto-Duel")
            except:
                self.wait_for_ui(1)
                continue
            self.wait_for_ui(.5)
        self.click_auto_duel()

    def wait_for_white_bottom(self, tryScanning=False):
        self.root.debug("WAITING FOR WHITE BOTTOM TO APPEAR")
        img = self.get_img_from_screen_shot()
        b = self.check_if_battle(img)
        while not b and not self.run_time.stop:
            if tryScanning:
                self.scan_for_word('ok', LOW_CORR)
            img = self.get_img_from_screen_shot()
            b = self.check_if_battle(img)
            if b:
                break
            self.wait_for_ui(1)

    def wait_for_ui(self, amount):
        if not self.run_time.stop:
            super(Provider, self).wait_for_ui(amount)


    def do_system_call(self, command):
        if not self.run_time.stop:
            CREATE_NO_WINDOW = 0x08000000
            subprocess.call(command, shell=True, creationflags=CREATE_NO_WINDOW)

    @staticmethod
    def img_to_string(img, char_set=None):
        cv2.imwrite("tmp\\ocr.png", img)
        command = "bin\\tess\\tesseract.exe --tessdata-dir bin\\tess\\tessdata tmp\\ocr.png tmp\\ocr "
        if char_set is not None:
            command += "-c tessedit_char_whitelist=" + char_set + " "
        command += "-psm 7 "
        command += "> nul 2>&1"
        CREATE_NO_WINDOW = 0x08000000
        subprocess.call(command, shell=True, creationflags=CREATE_NO_WINDOW)
        # Get the largest line in txt
        with open("tmp\\ocr.txt") as f:
            content = f.read().splitlines()
        output_line = ""
        for line in content:
            line = line.strip()
            if len(line) > len(output_line):
                output_line = line
        return output_line



import os as os

import cv2
import numpy as _np

from bot.providers.duellinks import Predefined
from bot.providers.shared import nox_current_version

left = 319
top = 79
width = 80
height = 23
auto_duel_box = (left, top, left + width, top + height)

duel_variant_v = {
    'v1': (230, 690),
    'v2-duel': (120, 690),
    'v2-autoduel': (290, 690)
}


class NoxPredefined(Predefined):
    files_need = [
        "auto_duel_on.png",
        "auto_duel_off.png",
        "new_duel_variant.png"
    ]

    def run_prechecks(self):
        for file in self.files_need:
            assert (os.path.exists(os.path.join(self.assets,
                                                file))), "Missing File for stats generations: if you git cloned this repo you probably have a miss configured home!!!"

    def generate(self):
        self.run_prechecks()
        save = {}
        temp_dict = self.generate_autoduel_stats()
        save = {**save, **temp_dict}
        temp_dict = self.generate_duel_button_stats()
        save = {**save, **temp_dict}
        save['version'] = nox_current_version
        self.write_hdf5(save, self.dataset)

    @property
    def autoduel(self):
        return {
            'left': 319,
            'top': 79,
            'width': 80,
            'height': 23,
        }

    @property
    def duel_variant(self):
        return {
            'left': 210,
            'top': 680,
            'width': 265 - 210,
            'height': 710 - 680
        }

    def generate_autoduel_stats(self):
        location = self.assets
        autoduelon = os.path.join(location, "auto_duel_on.png")
        autodueloff = os.path.join(location, "auto_duel_off.png")
        a = self.get_image_stats(cv2.imread(autodueloff), **self.autoduel)
        b = self.get_image_stats(cv2.imread(autoduelon), **self.autoduel)
        save = {
            'auto_duel_off': a,
            'auto_duel_on': b
        }
        return save

    def generate_duel_button_stats(self):
        location = self.assets
        new_duel_variant = os.path.join(location, "new_duel_variant.png")
        im = cv2.imread(new_duel_variant, cv2.COLOR_BGR2GRAY)
        edges = cv2.Canny(im, 240, 255)
        a = self.get_image_stats(_np.array(edges), **self.duel_variant)
        save = {
            'duel_variant': a
        }
        return save



from unittest import TestCase

import os

from bot.providers import Nox
from bot.utils.common import default_config


class TestNox(TestCase):
    provider = None

    def setUp(self):
        os.environ['LOG_CFG'] = r'D:\Sync\OneDrive\Yu-gi-oh_bot\config.ini'
        self.provider = Nox(None, default_config(r'D:\Sync\OneDrive\Yu-gi-oh_bot'), None)

    def test_provider(self):
        with self.assertRaises(AssertionError) as context:
            provider = Nox(None, default_config(r'D:\Sync\OneDrive\Fake'), None)
        self.assertTrue('Missing File' in str(context.exception))

    def test_start_process(self):
        with self.assertRaises(FileNotFoundError) as context:
            self.provider.NoxPath = 'C:\\Nox\\Not\\Here'
            self.provider.start_process()

    def test_is_process_running(self):
        self.fail()

    def test_kill_process(self):
        self.fail()



import logging
import os
import time
from abc import abstractmethod

import watchdog
from watchdog.events import LoggingEventHandler
from watchdog.events import PatternMatchingEventHandler
from watchdog.observers import Observer


class WatchFile(PatternMatchingEventHandler):
    root = logging.getLogger("bot.watcher")

    def on_moved(self, event):
        super(WatchFile, self).on_moved(event)
        self.root.debug("File %s was just moved" % event.src_path)

    def on_created(self, event):
        super(WatchFile, self).on_created(event)
        self.root.debug("File %s was just created" % event.src_path)

    def on_deleted(self, event):
        super(WatchFile, self).on_deleted(event)
        self.root.debug("File %s was just deleted" % event.src_path)

    def on_modified(self, event):
        super(WatchFile, self).on_modified(event)
        self.root.debug("File %s was just modified" % event.src_path)

    def on_any_event(self, event):
        super(WatchFile, self).on_modified(event)
        self.notify_event(event)

    def notify_event(self, event):
        self.root.debug(str(event))
        self.event_notification(event)

    @abstractmethod
    def event_notification(self, event):
        raise NotImplementedError("event notification not ready")


class SyncWithFile(WatchFile):
    _observer = None

    @property
    def observer(self):
        return self._observer

    @observer.setter
    def observer(self, observer):
        self._observer = observer

    _file_observing = None

    @property
    def file_observing(self):
        return self._file_observing

    @file_observing.setter
    def file_observing(self, value):
        self._file_observing = value

    def event_notification(self, event):
        """ HANDLES ROUTING OF WATCHDOG EVENT TYPES, SOME EDITORS MOVE TO TEMP FILES TO WRITE"""
        # TODO LP Investigate other possible file modifications
        if isinstance(event, watchdog.events.FileModifiedEvent):
            self.settings_modified(event)
        elif isinstance(event, watchdog.events.FileMovedEvent):
            if event.dest_path == self.file_observing:
                self.settings_modified(event)
        else:
            self.root.debug("Event type {} is not handled".format(type(event)))

    @abstractmethod
    def settings_modified(self, events):
        raise NotImplementedError("settings_modified not implemented")

    def __init__(self, file, auto_start=False):
        self.watcher = WatchFile(patterns=[file])
        self.watcher.event_notification = self.event_notification
        self.file_observing = file
        if auto_start:
            self.start_observer()

    def start_observer(self):
        self.observer = Observer()
        self.observer.schedule(self.watcher, os.path.dirname(self.file_observing), recursive=False)
        self.observer.start()

    def stop_observer(self):
        self.observer.stop()


if __name__ == "__main__":
    data_file = r"D:\Sync\OneDrive\Yu-gi-oh_bot\run_at.json"
    syncer = SyncWithFile(data_file, auto_start=True)
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        syncer.stop_observer()
    syncer.observer.join()



from abc import abstractmethod

import numpy as np
import cv2
import time


class Actions(object):
    """ Defines Actions that must be implemented for each Provider"""

    def __init__(self):
        pass

    @abstractmethod
    def tap(self, x, y):
        raise NotImplementedError("tap not defined")

    @abstractmethod
    def swipe(self, x1, y1, x2, y2):
        raise NotImplementedError("swipe not defined")

    @abstractmethod
    def swipe_time(self, x1, y1, x2, y2, time_amount):
        raise NotImplementedError("swipe time not defined")

    @abstractmethod
    def key_escape(self):
        "Optional Key Escape"
        raise NotImplementedError("key_escape not defined")

    @abstractmethod
    def take_png_screenshot(self):
        raise NotImplementedError("take_png_screenshot not defined")

    def get_img_from_screen_shot(self):
        screen_shot = self.take_png_screenshot()
        nparr = np.fromstring(screen_shot, np.uint8)
        img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
        return img

    @abstractmethod
    def swipe_right(self, time_sleep=0):
        raise NotImplementedError("swipe right not defined")

    def tapnsleep(self, point, time_sleep):
        x, y = point
        self.tap(x, y)
        self.wait_for_ui(time_sleep)

    @abstractmethod
    def wait_for_ui(self, time_sleep):
        raise NotImplementedError("wait_for_ui not defined")



import datetime
import json

import h5py
import numpy as np

from bot.utils.common import DotDict

data_object = {
    'next_run_at': None,
    'last_run_at': None,
    'runnow': False,
    'stop': False
}
data_object = DotDict(data_object)
data_file_name = "run_at.json"
data_file = data_file_name


def set_data_file(file_path):
    global data_file
    data_file = file_path


def read_data_file(key=None):
    try:
        with open(data_file) as f:
            data = json.load(f, object_hook=date_hook)
            data = DotDict(data)
    except:
        data = DotDict(data_object)
    if key == None:
        return data
    if key != None and key in data:
        return data[key]
    return None


def read_json_file(file=data_file):
    try:
        with open(file) as f:
            data = json.load(f, object_hook=date_hook)
            return data
    except FileNotFoundError:
        return None


def datetime_handler(x):
    if isinstance(x, datetime.datetime):
        return x.isoformat()
    raise TypeError("Unknown type")


try_formats = ["%Y-%m-%dT%H:%M:%S.%f", "%Y-%m-%dT%H:%M:%S"]


def date_hook(json_dict):
    for (key, value) in json_dict.items():
        for try_format in try_formats:
            try:
                json_dict[key] = datetime.datetime.strptime(
                    value, try_format)
            except:
                pass
    return json_dict


def write_data_file(data, file=data_file):
    with open(file, 'w') as f:
        json.dump(data, f, sort_keys=True,
                  indent=4, separators=(',', ': '), default=datetime_handler)


"""
Pulled from:
https://codereview.stackexchange.com/questions/120802/recursively-save-python-dictionaries-to-hdf5-files-using-h5py
"""


def save_dict_to_hdf5(dic, filename, mode='w'):
    """
    ....
    """
    with h5py.File(filename, mode) as h5file:
        recursively_save_dict_contents_to_group(h5file, '/', dic)


def recursively_save_dict_contents_to_group(h5file, path, dic):
    """
    ....
    """
    for key, item in dic.items():
        if isinstance(item, (np.ndarray, np.int64, np.float64, str, bytes)):
            h5file[path + key] = item
        elif isinstance(item, dict):
            recursively_save_dict_contents_to_group(h5file, path + key + '/', item)
        else:
            raise ValueError('Cannot save %s type' % type(item))


def load_dict_from_hdf5(filename):
    """
    ....
    """
    with h5py.File(filename, 'r') as h5file:
        return recursively_load_dict_contents_from_group(h5file, '/')


def recursively_load_dict_contents_from_group(h5file, path):
    """
    ....
    """
    ans = {}
    for key, item in h5file[path].items():
        if isinstance(item, h5py._hl.dataset.Dataset):
            ans[key] = item.value
        elif isinstance(item, h5py._hl.group.Group):
            ans[key] = recursively_load_dict_contents_from_group(h5file, path + key + '/')
    return ans


if __name__ == '__main__':
    data = {'x': 'astring',
            'y': np.arange(10),
            'd': {'z': np.ones((2, 3)),
                  'b': b'bytestring'}}
    print(data)
    filename = 'test.h5'
    save_dict_to_hdf5(data, filename)
    dd = load_dict_from_hdf5(filename)
    print(dd)



from unittest import TestCase

from watchdog.observers import Observer

from bot import last_record
from bot.utils.data import read_data_file, write_data_file, set_data_file
from bot.utils.watcher import WatchFile


class TestWatchFile(TestCase):
    """Tests for `watcher.py`."""
    watcher = None
    data_file = r"D:\Sync\OneDrive\Yu-gi-oh_bot\run_at.json"
    data = None

    def setUp(self):
        set_data_file(self.data_file)
        self.watcher = WatchFile(patterns=[self.data_file])
        self.observer = Observer()
        self.observer.schedule(self.watcher, r"D:\Sync\OneDrive\Yu-gi-oh_bot", recursive=False)
        self.observer.start()
        self.data = read_data_file()
        self.data['test'] = 'yes'

    def test_notify_event(self):
        write_data_file(self.data)
        records = [str(last_record.get_record(x)) for x in range(0, 5)]
        self.assertTrue("FileModifiedEvent" in ' '.join(records), "expecting a file modification event")

    def tearDown(self):
        self.observer.stop()



import os
import time
from abc import abstractmethod
from datetime import datetime

import cv2
import numpy as np

from bot import default_timestamp
from bot.utils.data import load_dict_from_hdf5, save_dict_to_hdf5
from .shared import *


class Event(object):
    _name = None
    _args = []
    _kwargs = {}

    def __init__(self, func, *args, **kwargs):
        self._name = func
        self._args = args
        self._kwargs = kwargs

    @property
    def func(self):
        return self._name

    @property
    def args(self):
        return self._args

    @property
    def kwargs(self):
        return self._kwargs


class DuelLinksInfo(object):
    _x = None
    _y = None
    _page = None
    _status = None

    def __init__(self, x, y, page, status):
        self._x = x
        self._y = y
        self._page = page
        self._status = status

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def page(self):
        return self._page

    @property
    def status(self):
        return self._status

    @x.setter
    def x(self, value):
        self._x = value

    @y.setter
    def y(self, value):
        self._y = value

    @page.setter
    def page(self, value):
        self._page = value

    @status.setter
    def status(self, value):
        self._status = value


class EventExecutor(object):
    def __init__(self):
        pass

    def do_event(self, _event):
        exists = getattr(self, _event.name, False)
        if exists:
            func = getattr(self, _event.name)
            if not callable(func):
                return False
            func(**_event.args)
            return True
        return False


class Predefined(object):
    _config = None
    dataset = None
    version = None
    assets = None

    def __init__(self, config, version):
        self._config = config
        self.cache_file = config.get('locations', 'cache_file')
        self.dataset = self.__class__.__name__
        self.assets = config.get('locations', 'assets')
        self.version = version
        self.get_cache()
        self.check_cache()

    _cache = None
    _last_read = datetime.fromtimestamp(default_timestamp)

    @property
    def cache(self):
        return self._cache

    @cache.setter
    def cache(self, value):
        self._last_read = datetime.now()
        self._cache = value

    def check_cache(self):
        pass

    def get_cache(self):
        if not os.path.exists(self.cache_file):
            self.generate()
        if self.cache is None:
            self.cache = load_dict_from_hdf5(self.cache_file)
            """
            self.cache = h5py.File(self.cache_file)
            if self.dataset in self.cache.keys():
                df = self.cache.get(self.dataset)
                version = df.get('version', 0)
                if version == 0 or version != self.version:
                    self.generate()
                    self.cache = h5py.File(self.cache_file)
                """

    _duel_varient = None

    @property
    def duel_variant(self):
        raise NotImplementedError("Class {} did not implement duel variant property".format(self.__class__.__name__))

    _auto_duel = None

    @property
    def autoduel(self):
        raise NotImplementedError("Class {} did not implement auto duel property".format(self.__class__.__name__))

    # TODO: IMPLEMENT METHOD TO DETERMINE THE ACCURACY OR THE LIKELHOOD THAT THIS IS AN AUTODUEL BUTTON


    def determine_autoduel_status(self, img):
        vals = self.cache.get(self.dataset)
        autodueloff = vals['auto_duel_off']
        autoduelon = vals['auto_duel_on']
        current = self.get_image_stats(img, **self.autoduel)
        dist1 = np.linalg.norm(current - autoduelon)
        dist2 = np.linalg.norm(current - autodueloff)
        if dist1 < dist2:
            return True
        return False

    def determine_duel_variant(self, img):
        vals = self.cache.get(self.dataset)
        ver_duel_variant = vals['duel_variant']
        edges = cv2.Canny(img, 240, 255)
        current = Predefined.get_image_stats(edges, **self.duel_variant)
        dist1 = np.linalg.norm(current - ver_duel_variant)
        if dist1 <= 5:
            return True
        return False

    @staticmethod
    def get_image_stats(img, left=0, top=0, width=0, height=0):
        crop_img = img[top:(top + height), left:(left + width)]
        (means, stds) = cv2.meanStdDev(crop_img)
        stats = np.concatenate([means, stds]).flatten()
        return stats

    def write_hdf5(self, data, dataset):
        data = {dataset: data}
        save_dict_to_hdf5(data, self.cache_file, mode='a')

    @abstractmethod
    def generate(self):
        raise NotImplementedError("Class {} did not implement generate".format(self.__class__.__name__))


class DuelLinks(object):
    _thread = None

    @property
    def current_thread(self):
        return self._thread

    @current_thread.setter
    def current_thread(self, thread):
        self.register_thread(thread)

    def register_thread(self, thread):
        self._thread = thread

    _auto_duel_box = None

    @property
    def auto_duel_box(self):
        "Determines the location of where the auto duel button is"
        return self._auto_duel_box

    _current_run = 0

    @property
    def current_run(self):
        return self._current_run

    @current_run.setter
    def current_run(self, run):
        self._current_run = run

    _sleep_factor = 1

    @property
    def sleep_factor(self):
        return self._sleep_factor

    @sleep_factor.setter
    def sleep_factor(self, value):
        self.sleep_factor = value

    def wait_for_ui(self, amount):
        time.sleep(amount * self.sleep_factor)

    @abstractmethod
    def auto(self):
        raise NotImplementedError("auto not implemented")

    @abstractmethod
    def debug_battle(self):
        "Helps to debug when battle it not going right"
        raise NotImplementedError("debug_battle not implemented")

    @abstractmethod
    def check_battle_is_running(self):
        raise NotImplementedError("check_battle_is_running not implemented")

    @abstractmethod
    def check_battle(self):
        raise NotImplementedError("check_battle not implemented")

    @abstractmethod
    def scan(self):
        raise NotImplementedError("scan not implemented")

    @abstractmethod
    def method_name(self):
        raise NotImplementedError("method_name not implemented")

    @abstractmethod
    def compare_with_back_button(self, corr=HIGH_CORR, info=None):
        raise NotImplementedError("compare_with_back_button not implemented")

    @abstractmethod
    def scan_for_word(self, word, corr=HIGH_CORR, log=None):
        raise NotImplementedError("scan_for_work not implemented")

    @abstractmethod
    def scan_for_close(self, corr=HIGH_CORR, log=None):
        raise NotImplementedError("scan_for_close not implemented")

    @abstractmethod
    def get_current_page(self, img):
        raise NotImplementedError("get_current_image not implemented")

    @abstractmethod
    def click_auto_duel(self):
        raise NotImplementedError("click_auto_duel not implemented")

    @abstractmethod
    def determine_autoduel_status(self):
        raise NotImplementedError("determine_autoduel_status not implemented")

    @abstractmethod
    def battle(self, check_battle=None, info=None):
        raise NotImplementedError("battle not implemented")

    @abstractmethod
    def check_if_battle(self, img):
        raise NotImplementedError("check_if_battle not implemented")

    @abstractmethod
    def verify_battle(self):
        raise NotImplementedError("verify_battle not implemented")

    @abstractmethod
    def pass_through_initial_screen(self):
        raise NotImplementedError("pass_through_initial_screen not implemented")

    @abstractmethod
    def wait_for(self, word, try_scanning=False):
        raise NotImplementedError("wait_for not implemented")

    @abstractmethod
    def wait_for_auto_duel(self):
        raise NotImplementedError("wait_for_auto_duel not implemented")

    @abstractmethod
    def wait_for_white_bottom(self):
        raise NotImplementedError("wait for white bottom not implemented")



#!/usr/bin/env python


#############################################################################
##
# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
# All rights reserved.
##
# This file is part of the examples of PyQt.
##
# $QT_BEGIN_LICENSE:BSD$
# You may use this file under the terms of the BSD license as follows:
##
# "Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in
# the documentation and/or other materials provided with the
# distribution.
# * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
# the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written
# permission.
##
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
# $QT_END_LICENSE$
##
#############################################################################
import time
from PyQt5 import QtCore
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QAction, QApplication, QCheckBox, QComboBox,
                             QDialog, QGridLayout, QGroupBox, QHBoxLayout, QLabel, QLineEdit,
                             QMessageBox, QMenu, QPushButton, QSpinBox, QStyle, QSystemTrayIcon,
                             QTextEdit, QVBoxLayout, QDesktopWidget, QWidget, QFrame)
from enum import Enum
from bot.duel_links_runtime import DuelLinkRunTime


class WINDOWS_TASKBAR_LOCATION(Enum):
    LEFT = 1
    TOP = 2
    RIGHT = 3
    BOTTOM = 4


app_name = "Yugioh-DuelLinks Bot"
default_open_offset = 7


def mock_data(): return False


update_intervals = {
    'next_run_at': 10,
    'nox_status': 10,
    'current_time': 1
}


class DuelLinksGui(QFrame):
    _shouldShowSystrayBox = mock_data
    dlRunTime = None

    def __init__(self, duelLinksRunTime=None):
        super(DuelLinksGui, self).__init__()
        # if duelLinksRunTime is None:
        #    raise Exception("Duel Links Run Time Invalid")
        self.dlRunTime = duelLinksRunTime  # type: DuelLinkRunTime
        # self.createIconGroupBox()
        self.createRunTimeFields()
        self.createMessageGroupBox()
        self.createBotControls()

        # self.setFrameStyle(QFrame.StyledPanel | QFrame.Sunken)
        # self.setLineWidth(2)
        self.setObjectName("BotFrame")
        self.setStyleSheet("#BotFrame {border: 2px solid #9e3939;}")
        # self.iconLabel.setMinimumWidth(self.durationLabel.sizeHint().width())

        self.createActions()
        self.createBotActions()
        self.createTrayIcon()
        self.setShouldShowSystrayBox(mock_data)
        # self.showMessageButton.clicked.connect(self.showMessage)
        self.hideButton.clicked.connect(self.close)
        self.exitButton.clicked.connect(self.__quit__)
        # self.showIconCheckBox.toggled.connect(self.trayIcon.setVisible)
        # self.iconComboBox.currentIndexChanged.connect(self.setIcon)
        self.trayIcon.messageClicked.connect(self.messageClicked)
        self.trayIcon.activated.connect(self.iconActivated)

        # bot actions connected

        self.pauseButton.clicked.connect(self.pause_bot)
        self.runButton.clicked.connect(self.start_bot)

        mainLayout = QVBoxLayout()
        mainLayout.addWidget(self.runTimeGroupBox)
        mainLayout.addWidget(self.botControls)
        # mainLayout.addWidget(self.messageGroupBox)
        self.setLayout(mainLayout)

        self.setIcon(0)
        self.trayIcon.show()
        self.setWindowTitle(app_name)
        self.setFixedSize(400, 300)
        self.setWindowFlags(QtCore.Qt.FramelessWindowHint | QtCore.Qt.Popup)
        self.location_on_the_screen()
        self.update_values(True)

    def location_on_the_screen(self):
        ag = QDesktopWidget().availableGeometry()
        sg = QDesktopWidget().screenGeometry()

        widget = self.geometry()
        position = self.get_task_bar_position()
        if position == WINDOWS_TASKBAR_LOCATION.BOTTOM:
            x = ag.width() - widget.width()
            y = 2 * ag.height() - sg.height() - widget.height()
        elif position == WINDOWS_TASKBAR_LOCATION.LEFT:
            x = sg.width() - ag.width() + default_open_offset
            y = 2 * ag.height() - sg.height() - widget.height() - default_open_offset
        self.move(x, y)

    def get_task_bar_position(self):
        desktop = QDesktopWidget()
        displayRect = desktop.screenGeometry()
        desktopRect = desktop.availableGeometry()
        if desktopRect.height() < displayRect.height():
            if desktopRect.y() > displayRect.y():
                return WINDOWS_TASKBAR_LOCATION.TOP
            else:
                return WINDOWS_TASKBAR_LOCATION.BOTTOM
        else:
            if desktopRect.x() > displayRect.x():
                return WINDOWS_TASKBAR_LOCATION.LEFT
            else:
                return WINDOWS_TASKBAR_LOCATION.RIGHT

    def setVisible(self, visible):
        self.minimizeAction.setEnabled(visible)
        self.maximizeAction.setEnabled(not self.isMaximized())
        self.restoreAction.setEnabled(self.isMaximized() or not visible)
        super(DuelLinksGui, self).setVisible(visible)

    def closeEvent(self, event):
        if self.trayIcon.isVisible():
            if self.shouldShowSystrayBox():
                QMessageBox.information(self, app_name,
                                        "The program will keep running in the system tray. To "
                                        "terminate the program, choose <b>Quit</b> in the "
                                        "context menu of the system tray entry.")
            self.hide()
            event.ignore()

    def setShouldShowSystrayBox(self, callback):
        self._shouldShowSystrayBox = callback

    def shouldShowSystrayBox(self):
        self._shouldShowSystrayBox()

    def setIcon(self, index):
        icon = QIcon('assets/yugioh.ico')
        self.trayIcon.setIcon(icon)
        self.setWindowIcon(icon)

        self.trayIcon.setToolTip('Duel-Links Bot')

    def iconActivated(self, reason):
        if reason in (QSystemTrayIcon.Trigger, QSystemTrayIcon.DoubleClick):
            self.showNormal()
        elif reason == QSystemTrayIcon.MiddleClick:
            self.showNotifcation(
                "In Development", "You pressed the middle mouse button.\n Hidden Feature!!!!")

    def showMessage(self):
        icon = QSystemTrayIcon.MessageIcon(
            self.typeComboBox.itemData(self.typeComboBox.currentIndex()))
        self.trayIcon.showMessage(self.titleEdit.text(),
                                  self.bodyEdit.toPlainText(), icon,
                                  self.durationSpinBox.value() * 1000)

    def showNotifcation(self, title, message):
        icon = QSystemTrayIcon.MessageIcon(
            self.typeComboBox.itemData(self.typeComboBox.currentIndex()))
        self.trayIcon.showMessage(title,
                                  message, icon,
                                  self.durationSpinBox.value() * 1000)

    def messageClicked(self):
        QMessageBox.information(None, "Systray",
                                "Sorry, I already gave what help I could.\nMaybe you should "
                                "try asking a human?")

    def createIconGroupBox(self):
        self.iconGroupBox = QGroupBox("Tray Icon")

        self.iconLabel = QLabel("Icon:")

        self.iconComboBox = QComboBox()
        self.iconComboBox.addItem(QIcon('assets/yugioh.ico'), "Duel-Card")

        self.showIconCheckBox = QCheckBox("Show icon")
        self.showIconCheckBox.setChecked(True)

        iconLayout = QHBoxLayout()
        iconLayout.addWidget(self.iconLabel)
        iconLayout.addWidget(self.iconComboBox)
        iconLayout.addStretch()
        iconLayout.addWidget(self.showIconCheckBox)
        self.iconGroupBox.setLayout(iconLayout)

    def createBotControls(self):
        self.botControls = QGroupBox("Controls")
        controlLayout = QGridLayout()
        runLabel = QLabel("Run the bot:")
        self.runButton = QPushButton("Run")
        showLabel = QLabel("Pause the bot:")
        self.pauseButton = QPushButton("Pause")
        self.exitButton = QPushButton("Exit")
        self.hideButton = QPushButton("Hide")
        controlLayout.addWidget(runLabel, 0, 0)
        controlLayout.addWidget(self.runButton, 0, 2, 1, 2)
        controlLayout.addWidget(showLabel, 1, 0)
        controlLayout.addWidget(self.pauseButton, 1, 2, 1, 2)
        controlLayout.addWidget(self.hideButton, 2, 0, 1, 2)
        controlLayout.addWidget(self.exitButton, 2, 2, 1, 2)
        self.botControls.setLayout(controlLayout)

    def createMessageGroupBox(self):
        # self.messageGroupBox = QGroupBox("Balloon Message")

        typeLabel = QLabel("Type:")

        self.typeComboBox = QComboBox()
        self.typeComboBox.addItem("None", QSystemTrayIcon.NoIcon)
        self.typeComboBox.addItem(self.style().standardIcon(
            QStyle.SP_MessageBoxInformation), "Information",
            QSystemTrayIcon.Information)
        self.typeComboBox.addItem(self.style().standardIcon(
            QStyle.SP_MessageBoxWarning), "Warning",
            QSystemTrayIcon.Warning)
        self.typeComboBox.addItem(self.style().standardIcon(
            QStyle.SP_MessageBoxCritical), "Critical",
            QSystemTrayIcon.Critical)
        self.typeComboBox.setCurrentIndex(1)

        self.durationLabel = QLabel("Duration:")

        self.durationSpinBox = QSpinBox()
        self.durationSpinBox.setRange(5, 60)
        self.durationSpinBox.setSuffix(" s")
        self.durationSpinBox.setValue(15)

        durationWarningLabel = QLabel("(some systems might ignore this hint)")
        durationWarningLabel.setIndent(10)

        titleLabel = QLabel("Title:")

        self.titleEdit = QLineEdit("Cannot connect to network")

        bodyLabel = QLabel("Body:")

        self.bodyEdit = QTextEdit()
        self.bodyEdit.setPlainText("Don't believe me. Honestly, I don't have "
                                   "a clue.\nClick this balloon for details.")

        # self.showMessageButton = QPushButton("Show Message")
        # self.showMessageButton.setDefault(True)
        """
        messageLayout = QGridLayout()
        messageLayout.addWidget(typeLabel, 0, 0)
        messageLayout.addWidget(self.typeComboBox, 0, 1, 1, 2)
        messageLayout.addWidget(self.durationLabel, 1, 0)
        messageLayout.addWidget(self.durationSpinBox, 1, 1)
        messageLayout.addWidget(durationWarningLabel, 1, 2, 1, 3)
        messageLayout.addWidget(titleLabel, 2, 0)
        messageLayout.addWidget(self.titleEdit, 2, 1, 1, 4)
        messageLayout.addWidget(bodyLabel, 3, 0)
        messageLayout.addWidget(self.bodyEdit, 3, 1, 2, 4)
        messageLayout.addWidget(self.showMessageButton, 5, 4)
        messageLayout.setColumnStretch(3, 1)
        messageLayout.setRowStretch(4, 1)
        self.messageGroupBox.setLayout(messageLayout)"""

    def createRunTimeFields(self):
        self.runTimeGroupBox = QGroupBox("RunTime Fields")
        self.current_time = QLabel("Current Time: ")
        self.current_time_value = QLabel("")
        self.nox_status_label = QLabel("Nox status: ")
        self.nox_status_value = QLabel("")
        self.next_run_at_label = QLabel("Next Run At:")
        self.next_run_at_value = QLabel("")
        self.in_timer = QtCore.QTimer(self)
        self.in_timer.setInterval(1000)
        self.in_timer.timeout.connect(self.update_values)
        self.in_timer.start()
        layout = QVBoxLayout()
        top = QHBoxLayout()
        top.addWidget(self.current_time)
        top.addWidget(self.current_time_value)
        top.addStretch()
        runTimeLayout = QHBoxLayout()
        runTimeLayout.addWidget(self.nox_status_label)
        runTimeLayout.addWidget(self.nox_status_value)
        runTimeLayout.addStretch()
        runTimeLayout.addWidget(self.next_run_at_label)
        runTimeLayout.addWidget(self.next_run_at_value)
        layout.addLayout(top)
        layout.addLayout(runTimeLayout)
        self.runTimeGroupBox.setLayout(layout)

    _counter = 0

    def update_values(self, force=False):
        self._counter += 1
        if self._counter % update_intervals.get('current_time', 1) == 0 or force:
            self.current_time_value.setText(QtCore.QDateTime.currentDateTime().toString())
        if self._counter % update_intervals.get('nox_status', 1) == 0 or force:
            self.nox_status_value.setText(
                (lambda: "Running" if self.dlRunTime.get_provider().is_process_running() else "Off")())
        if self._counter % update_intervals.get('next_run_at', 1) == 0 or force:
            self.next_run_at_value.setText(self.dlRunTime.next_run_at.strftime("%Y-%m-%dT%H:%M:%S"))
        if self.dlRunTime.get_provider().current_thread is not None:
            self.runButton.setDisabled(False)
            self.runButton.setEnabled(False)
            self.pauseButton.setDisabled(True)
            self.pauseButton.setEnabled(True)
        else:
            self.runButton.setDisabled(True)
            self.runButton.setEnabled(True)
            self.pauseButton.setDisabled(False)
            self.pauseButton.setEnabled(False)

    def createActions(self):
        self.minimizeAction = QAction("Mi&nimize", self, triggered=self.hide)
        self.maximizeAction = QAction("Ma&ximize", self,
                                      triggered=self.showMaximized)
        self.restoreAction = QAction("&Restore", self,
                                     triggered=self.showNormal)
        self.quitAction = QAction("&Quit", self,
                                  triggered=self.__quit__)

    def __quit__(self):
        self.hide()
        self.dlRunTime.shutdown()
        QApplication.instance().quit()

    def createBotActions(self):
        self.startAction = QAction('Start', self, triggered=self.start_bot)
        self.pauseAction = QAction('Pause', self, triggered=self.pause_bot)

    def start_bot(self):
        self.dlRunTime.stop = False
        self.dlRunTime.run_now = True

    def pause_bot(self):
        self.dlRunTime.stop = True
        self.dlRunTime.run_now = False

    def createTrayIcon(self):
        self.trayIconMenu = QMenu(self)
        self.trayIconMenu.addAction(self.minimizeAction)
        self.trayIconMenu.addAction(self.maximizeAction)
        self.trayIconMenu.addAction(self.restoreAction)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.quitAction)

        self.trayIcon = QSystemTrayIcon(self)
        self.trayIcon.setContextMenu(self.trayIconMenu)


if __name__ == '__main__':

    import sys

    app = QApplication(sys.argv)

    if not QSystemTrayIcon.isSystemTrayAvailable():
        QMessageBox.critical(None, "Systray",
                             "Systray not dected on system.")
        sys.exit(1)

    QApplication.setQuitOnLastWindowClosed(False)

    window = Window()
    window.show()
    sys.exit(app.exec_())



from abc import abstractmethod


class Misc(object):
    @abstractmethod
    def is_process_running(self):
        raise NotImplementedError("is_process_running not implemented")

    @abstractmethod
    def start_process(self):
        raise NotImplementedError("start_process not implemented")

    @abstractmethod
    def kill_process(self):
        raise NotImplementedError("kill_process not implemented")



from setuptools import setup

import versioneer

DISTNAME = 'yugioh-bot'
AUTHOR = 'will7200'
EMAIL = 'yugiohbotgit@gmail.com'
LICENSE = 'MIT'
URL = 'https://github.com/will7200/Yugioh-bot'
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Operating System :: Microsoft :: Windows',
    'Intended Audience :: End Users/Desktop',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: MIT License']
setup(
    name=DISTNAME,
    maintainer=AUTHOR,
    version=versioneer.get_version(),
    packages=[
        'bot',
        'bot.providers',
        'bot.providers.nox',
        'bot.utils'
    ],
    cmdclass=versioneer.get_cmdclass(),
    classifiers=CLASSIFIERS,
    platforms='win32',
    maintainer_email=EMAIL,
    python_requires='>=3.5',
    url=URL,
    description="Botting Yugioh-DuelLinks",
    long_description=open('README.md', 'rt').read(),
    keywords='yugioh bot yugioh-bot duellinks duel links yugioh-duellinks'
)



import base64
import logging
import os
import subprocess
import sys
import win32gui

import cv2
import numpy as np

from bot.providers import trainer_matches as tm
from bot.providers.common import loop_scan, crop_image, mask_image
from bot.providers.duellinks import DuelLinksInfo
from bot.providers.nox.predefined import NoxPredefined, duel_variant_v
from bot.providers.provider import Provider
from bot.providers.shared import *
from bot.debug_helpers import calling_function


class Nox(Provider):
    NotPath = None

    def setUp(self):
        super(Nox, self).setUp()
        self.predefined = NoxPredefined(self._config, nox_current_version)
        self.NoxPath = os.path.join(self._config.get('bot', 'noxlocation'), 'Nox.exe')

    def swipe_time(self, x1, y1, x2, y2, time_amount):
        command = "bin\\adb.exe shell input swipe %d %d %d %d %d" % (
            x1, y1, x2, y2, time_amount)
        self.do_system_call(command)

    def swipe_right(self, time_sleep=0):
        self.swipe(0, 500, 100, 500)
        self.wait_for_ui(2)

    def swipe(self, x1, y1, x2, y2):
        command = "bin\\adb.exe shell input swipe %d %d %d %d " % (x1, y1, x2, y2)
        self.do_system_call(command)

    def take_png_screenshot(self):
        while True:
            try:
                command = "bin\\adb.exe shell \"screencap -p | busybox base64\""
                png_screenshot_data = os.popen(command).read()
                png_screenshot_data = base64.b64decode(png_screenshot_data)
                break
            except KeyboardInterrupt:
                sys.exit(0)
            except:
                print("[!] Failed to get screen")
        return png_screenshot_data

    def tap(self, x, y):
        self.root.debug("Tapping at location ({},{})".format(x, y))
        command = "bin\\adb.exe shell input tap %d %d" % (x, y)
        self.do_system_call(command)

    def key_escape(self):
        command = "bin\\adb.exe shell input keyevent 4"
        self.do_system_call(command)

    root = logging.getLogger("bot.provider.Nox")

    @staticmethod
    def __str__():
        return "Nox"

    def wait_for(self, word, try_scanning=False):
        self.root.info("WAITING FOR {} BUTTON TO APPEAR".format(word))
        ok = ''
        while ok != word and not self.run_time.stop:
            # root.debug("waiting for {}".format(word))
            img = self.get_img_from_screen_shot()
            img = img[745:770, 210:270]
            try:
                if try_scanning:
                    self.scan_for_word('ok', LOW_CORR)
                ok = self.img_to_string(img,
                                        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
            except:
                self.wait_for_ui(1)
                continue
            if ok == word:
                break
            self.wait_for_ui(2)

    def pass_through_initial_screen(self):
        self.root.info("Passing Through Start Screen")
        self.wait_for_ui(30)
        self.tapnsleep((25, 550), 10)
        self.tapnsleep((240, 530), 45)

    def verify_battle(self):
        try_times = 3
        version = 0
        while True:
            self.root.debug("Verifying battle")
            try_times -= 1
            img = self.get_img_from_screen_shot()
            if self.predefined.determine_duel_variant(img):
                pointer = duel_variant_v['v2-autoduel']
                img = img[680:710, 300:420]
                version = 2
                break
            elif try_times == 0:
                pointer = duel_variant_v['v1']
                img = img[680:710, 210:265]
                version = 1
                break

        # img = img[680:710, 90:150] new version duel
        img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        ok = self.img_to_string(img,
                                "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz").lower()
        if version == 1:
            if ok.startswith("due") or ok == "duel":
                return (pointer, version)
        if version == 2:
            if ok.startswith("auto") or 'auto' in ok:
                return (pointer, version)
        return None

    def scan_for_close(self, corr=HIGH_CORR, info=None):
        corrword = 'HIGH' if corr == HIGH_CORR else 'LOW'
        self.root.debug("LOOKING FOR CLOSE BUTTON, {} CORRERLATION".format(corrword))
        img = self.get_img_from_screen_shot()
        t = tm.Trainer(img, 400, 500)
        location = os.path.join(self.assets, "close.png")
        return self.__wrapper_kmeans_result(t, location, corr, info)

    def method_name(self):
        super(Nox, self).method_name()

    def start_process(self):
        try:
            self.root.info("Starting Nox...")
            process = subprocess.Popen([self.NoxPath], stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)
        except FileNotFoundError as e:
            self.root.fatal("Nox executable not found")
            raise e
        except:
            self.root.error("The program can't run Nox")
            raise NotImplementedError

    def is_process_running(self):
        try:
            if win32gui.FindWindow(None, "Nox") or win32gui.FindWindow(None, "NoxPlayer"):
                return True
        except:
            return False

    def compare_with_back_button(self, corr=HIGH_CORR, info=None):
        corrword = 'HIGH' if corr == HIGH_CORR else 'LOW'
        self.root.debug("LOOKING FOR BACK BUTTON, {} CORRERLATION".format(corrword))
        img = self.get_img_from_screen_shot()
        t = tm.Trainer(img, 150, 720)
        location = os.path.join(self.assets, "back__.png")
        return self.__wrapper_kmeans_result(t, location, corr, info)

    def __wrapper_kmeans_result(self, trainer, location, corr, info=None):
        if trainer.getMatches(location, corr):
            x, y = trainer.kmeans.cluster_centers_[0]
            if info:
                self.root.info("NPC Battle Mode,Points: ({},{}) at location: ({}), message: {}".format(
                    info.x, info.y, info.page, info.status
                ))
            self.tap(x, y)
            return True
        return False

    def get_current_page(self, img):
        left = 0
        top = 775
        width = 480
        height = 25
        area = crop_image(img, left, top, width, height)
        area = mask_image([254, 254, 254], [255, 255, 255], area)
        height, width = area.shape
        current_page = 0
        for x in range(4):
            box = crop_image(area, (x * width / 4), 0, ((x + 1) * width / 4), height)
            height1, width1 = box.shape
            if (height1 * width1 - cv2.countNonZero(box)) > 0:
                current_page = x
        return current_page + 1

    def click_auto_duel(self):
        self.root.debug("AUTO-DUEL FOUND CLICKING")
        self.wait_for_ui(.1)
        self.tap(356, 85)

    def battle(self, info=None):
        "The main battle mode"
        self.wait_for('OK')
        if info:
            info.status = "Battle Ended"
            self.root.info("NPC Battle Mode,Points: ({},{}) at location: ({}), message: {}".format(
                info.x, info.y, info.page, info.status
            ))

        self.wait_for_ui(.5)
        self.tap(230, 750)
        self.wait_for('NEXT', True)
        self.tapnsleep((230, 750), .5)
        self.wait_for('NEXT')
        self.wait_for_ui(.3)
        self.tap(230, 750)
        self.wait_for_white_bottom(True)
        self.wait_for_ui(.5)
        self.tapnsleep((230, 750), .1)
        dialog = True
        while dialog:
            dialog = self.check_if_battle(self.get_img_from_screen_shot())
            if dialog:
                self.tap(230, 750)
        self.wait_for_ui(.5)
        self.scan_for_word('ok', LOW_CORR)
        self.wait_for_ui(.1)
        self.scan_for_word('ok', LOW_CORR)
        battle_calls = self.run_time.battle_calls
        for section in ["beforeStart", "afterStart", "beforeEnd", "afterEnd"]:
            for value in battle_calls.get(section):
                self.root.debug(value)

    def check_if_battle(self, img):
        img = np.array(img)
        img = img[750:800, 0:400]
        blue_min = np.array([250, 250, 250], np.uint8)
        blue_max = np.array([255, 255, 255], np.uint8)
        amount = cv2.inRange(img, blue_min, blue_max)
        if cv2.countNonZero(amount) > (50 * 200):
            return True
        return False

    def determine_autoduel_status(self):
        super(Nox, self).determine_autoduel_status()

    def check_battle_is_running(self):
        super(Nox, self).check_battle_is_running()

    def kill_process(self):
        try:
            if self.is_process_running():
                command = "taskkill /im Nox.exe /f"
                CREATE_NO_WINDOW = 0x08000000
                subprocess.call(command, shell=True, creationflags=CREATE_NO_WINDOW)
        except:
            self.root.error("The program could not be killed")

    def scan_for_word(self, word, corr=HIGH_CORR, info=None):
        corrword = 'HIGH' if corr == HIGH_CORR else 'LOW'
        self.root.debug("LOOK FOR WORD '{}', {} CORRERLATION".format(word, corrword))
        img = self.get_img_from_screen_shot()
        t = tm.Trainer(img, 480, 50)
        location = os.path.join(self.assets, "ok_box.png")
        return self.__wrapper_kmeans_result(t, location, corr, info)

    def scan(self):
        for x, y, current_page in self.possible_battle_points():
            self.compare_with_back_button(info=None)
            self.wait_for_ui(1)
            self.tapnsleep((x, y), .5)
            img1 = self.get_img_from_screen_shot()
            battle = self.check_if_battle(img1)
            self.wait_for_ui(2.5)
            dl_info = DuelLinksInfo(x, y, current_page, "Starting Battle")
            if battle:
                self.tapnsleep((150, 400), 2.5)
                battle = self.verify_battle()
            if battle:
                self.current_battle = True
                self.root.info(battlemode % (x, y, current_page, "Starting Battle"))
                self.scan_for_word('ok', LOW_CORR)
                p, v = battle
                self.tapnsleep(p, 0)
                self.battle(dl_info)
            else:
                self.wait_for_ui(2)
                loop_scan(self.compare_with_back_button, **{'info': dl_info})
                dl_info.status = "failure/closeButton"
                loop_scan(self.scan_for_close, **{'info': dl_info})
                dl_info.status = "success/Gift"
                loop_scan(self.scan_for_word, **{'word': 'ok', 'info': dl_info})
            self.wait_for_ui(2)



import configparser

from os.path import expanduser, join


def rotate_list(l, n):
    return l[-n:] + l[:-n]


class DotDict(dict):
    """dot.notation access to dictionary attributes"""
    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


def default_config(home="."):
    """Make the default configuration"""
    config = configparser.ConfigParser()

    # Directories to read from
    config['locations'] = {
        'home': join(home),
        'assets': join(home, "assets"),
        'bin': join(home, "bin"),
        'log': join(home, "log"),
        'cache_file': join(home, "assets", "predefined.h5")
    }

    # Bot config values
    config['bot'] = {
        'killNoxOnDone': True,  # kill nox when a run through is done
        'noxLocation': r'C:\Program Files (x86)\Nox\bin',  # nox location
        'startBotOnStartUp': False,  # start bot irregardless of next run time
        'runTimePersistence': 'runTimeOptions.json',  # store runtime options to this file
        'provider': 'Choose'
    }
    return config


def make_config_file(file):
    """Make the default configuration file"""
    config = default_config()
    with open(file, 'w') as f:
        config.write(f)
